/*

Copyright (c) 2005 - 2009. All Rights reserved.

If you use this script, please email me and let me know, thanks!

Andrew Hedges
andrew (at) hedges (dot) name

If you want to hire me to write JavaScript for you, see my resume.

http://andrew.hedges.name/resume/

*/

// The following function has been "borrowed" from DashCode
// localize() pulls a string out an array named LocalizedStrings. Each language project
// directory in this widget contains a file named "LocalizedStrings.js", which, in turn, contains
// an array called localizedStrings. This method queries the array of the file of whichever
// language has highest precidence, according to the International pane of System Preferences.
String.prototype.localize = function () {
	var ret;
	try {
		ret = LocalizedStrings[this];
		if ('undefined' !== typeof ret) return ret;
	} catch (err) {}
	return this;
};

// Convenience method for String.prototype.localize
// Usage: __('String to be localized')
__ = function (str) {
	return str.localize();
};

var WW, GETDILBERT;

WW = window.widget;

GETDILBERT = (function () {
	// "constants"
	var GET, ITEM, NONE, BLOCK, NONET, EMPTY, LOADED, LOADING, NOCACHE, NOTFOUND, UNDEFINED, LASTLOADED, DESCRIPTION, CACHECONTROL;
	GET          = 'GET';
	DASH         = '-';
	ITEM         = 'item';
	NONE         = 'none';
	BLOCK        = 'block';
	NONET        = 'nonet';
	EMPTY        = 'empty';
	WHITE        = 'white';
	TOBACK       = 'ToBack';
	LOADED       = 'loaded';
	LOADING      = 'loading';
	TOFRONT      = 'ToFront';
	NOCACHE      = 'no-cache';
	NOTFOUND     = 'notfound';
	UNDEFINED    = 'undefined';
	LASTLOADED   = 'lastLoaded';
	INLINEBLOCK  = 'inline-block';
	DESCRIPTION  = 'description';
	CACHECONTROL = 'Cache-Control';
	// private variables
	var _feed, _rgxp, _interval, _defaults, _msgs, _dogberts, _dogbertError, _dogbertGo, _elements, _timer, _xhr, _version;
	_feed = 'http://feeds.feedburner.com/DilbertDailyStrip';
	_rgxp = /src="(http.*[\.gif|\.jpg])"/i;
	_interval = 135;
	_frame    = 0;
	_defaults = {
		width  : 468,
		height : 237
	};
	_msgs = {
		loading  : __('Comic loading. Please wait&#8230;'),
		loaded   : __('Ready! Click Dogbert&#8230;'),
		nonet    : __('No internet connection.'),
		empty    : __('Empty page.'),
		notfound : __('Comic not found. Sorry!')
	};
	_dogberts = [
		'Assets/imgs/0-dogbert.gif',
		'Assets/imgs/1-dogbert.gif',
		'Assets/imgs/2-dogbert.gif',
		'Assets/imgs/3-dogbert.gif'
	];
	_dogbertError = 'Assets/imgs/dogbert-error.gif';
	_dogbertGo    = 'Assets/imgs/dogbert-go.gif'
	_elements = {
		front    : 'front',
		dfault   : 'default',
		back     : 'back',
		dogbert  : 'dogbert',
		msg      : 'msg',
		comic    : 'comic',
		comicDiv : 'comicDiv',
		flipper  : 'flipper',
		done     : 'done',
		version  : 'new-version'
	};
	// private methods
	var _onHide, _onShow, _dogbert, _now, _generateUniqueKey, _setPref, _getPref, _displayStatus, _getComic, _makeRequest, _onStateChange, _processResult, _activateDogbert, _showComic, _hideComic, _getLocalVersion, _checkVersion;
	_onHide = function () {
		if (UNDEFINED !== typeof _xhr.abort) {
			_xhr.abort();
		}
		_dogbert.stop();
	};
	_onShow = function () {
		_getComic();
	};
	_dogbert = {
		start: function () {
			_timer = setInterval(function () {
				_elements.dogbert.src = _dogberts[_frame];
				++_frame;
				if (_frame == _dogberts.length) _frame = 0;
			}, _interval);
		},
		stop: function () {
			clearInterval(_timer);
		}
	};
	_now = function () {
		return (new Date()).getTime();
	};
	_generateUniqueKey = function (key) {
		var uniqueKey;
		uniqueKey = WW.identifier + DASH + key;
		return uniqueKey;
	};
	_setPref = function (key, value) {
		var uniqueKey;
		uniqueKey = _generateUniqueKey(key);
		WW.setPreferenceForKey(value, uniqueKey);
	};
	_getPref = function (key) {
		var uniqueKey, val;
		uniqueKey = _generateUniqueKey(key);
		val       = WW.preferenceForKey(uniqueKey);
		return (UNDEFINED === typeof val)? false : val;
	};
	_displayStatus = function (key) {
		_elements.msg.innerHTML = _msgs[key];
	};
	_getComic = function () {
		var lastLoaded;
		_displayStatus(LOADING);
		_dogbert.start();
		lastLoaded = _getPref(LASTLOADED);
		// if it's been more than an hour since we loaded a new comic, do it
		if (!lastLoaded || _now() - lastLoaded > 3600000) {
			_makeRequest();
		} else {
			_displayStatus(LOADED);
			_activateDogbert();
		}
	};
	_makeRequest = function () {
		_xhr = new XMLHttpRequest();
		_xhr.setRequestHeader(CACHECONTROL, NOCACHE);
		_xhr.onreadystatechange = _onStateChange;
		_xhr.open(GET, _feed, true);
		_xhr.send(null);
	};
	_onStateChange = function () {
		if (null === _xhr.readyState) return;
		if (4 === _xhr.readyState) {
			_dogbert.stop();
			if (_xhr.status < 400) {
				_processResult();
			} else {
				if (null === _xhr.status) return;
				_displayStatus(NONET);
				_elements.dogbert.src = _dogbertError;
			}
		}
	};
	_processResult = function () {
		var item, desc, imgSrc;
		if (null === _xhr.responseText) {
			_displayStatus(EMPTY);
			_elements.dogbert.src = _dogbertError;
		} else {
			
/* 			DEBUG.write(_xhr.responseText); */
			
			item   = _xhr.responseXML.getElementsByTagName(ITEM)[0];
			desc   = item.getElementsByTagName(DESCRIPTION)[0];
			imgSrc = desc.firstChild.nodeValue.match(_rgxp)[1];
			_elements.comic.src = imgSrc;
			_setPref(LASTLOADED, _now());
			_displayStatus(LOADED);
			_activateDogbert();
		}
	};
	_activateDogbert = function () {
		_dogbert.stop();
		_elements.dogbert.src     = _dogbertGo;
		_elements.dogbert.onclick = _showComic;
	};
	_hideComic = function () {
		_elements.comicDiv.style.display = NONE;
		_elements.dfault.style.display   = BLOCK;
	};
	_showComic = function () {
		_elements.comicDiv.style.display = INLINEBLOCK;
		_elements.dfault.style.display   = NONE;
	};
	_getLocalVersion = function () {
		_version = '1.8';
	};
	_checkVersion = function () {
		var url, xhr, onStateChange, processResult;
		url = 'http://segdeha.com/widgets/version.php?widget=getdilbert';
		onStateChange = function () {
			if (null === xhr.readyState) return;
			if (4 === xhr.readyState) {
				if (xhr.status < 400) {
					processResult();
				}
			}
		};
		processResult = function () {
			if (null !== xhr.responseText && 'undefined' !== typeof _version) {
				if (_version < xhr.responseText) {
					_elements.version.style.display = BLOCK;
				} else {
					_elements.version.style.display = NONE;
				}
			}
		};
		xhr = new XMLHttpRequest();
		xhr.setRequestHeader(CACHECONTROL, NOCACHE);
		xhr.onreadystatechange = onStateChange;
		xhr.open(GET, url, true);
		xhr.send(null);
	};
	return {
		init: function () {
			var flipper, done;
			window.resizeTo(600, 400);
			WW.onhide = _onHide;
			WW.onshow = _onShow;
			for (key in _elements) {
				_elements[key] = document.getElementById(_elements[key]);
			}
			flipper = new AppleInfoButton(_elements.flipper, _elements.dfault, WHITE, WHITE, GETDILBERT.showBack);
			done    = new AppleGlassButton(_elements.done, __('Done'), GETDILBERT.showFront);
			_getComic();
			_getLocalVersion();
		},
		refresh: function () {
			_hideComic();
			_makeRequest();
		},
		showBack: function () {
			_checkVersion();
			WW.prepareForTransition(TOBACK);
			_elements.front.style.display = NONE;
			_elements.back.style.display  = BLOCK;
			setTimeout(function () {
				WW.performTransition();
			}, 0);
		},
		showFront: function () {
			WW.prepareForTransition(TOFRONT);
			_elements.front.style.display = BLOCK;
			_elements.back.style.display  = NONE;
			setTimeout(function () {
				WW.performTransition();
			}, 0);
		}
	};
})();

window.onload = GETDILBERT.init;

/**
 * Debugging functions that work for both strings and objects in both widget and browser environs
 */
var DEBUG;
DEBUG = (function () {
	var _startTime, _elapsedTime;
	_startTime   = 0;
	_elapsedTime = 0;
	return {
		write: function (s) {
			alert('----------------------------------------------------------');
			alert(s);
		},
		reveal: function (o, showFunctions) {
			for (p in o) {
				if (showFunctions || 'function' !== typeof o[p]) {
					DEBUG.write(p + ': ' + o[p]);
				}
			}
		},
		startTimer: function () {
			_startTime = (new Date()).getTime();
		},
		markTime: function () {
			_elapsedTime = ((new Date()).getTime()) - _startTime;
		},
		getElapsedTime: function () {
			return (_elapsedTime > 0)? _elapsedTime : 0;
		}
	};
})();
